﻿using System;
using System.IO;
using System.Web;
using System.Net;
using System.Text;
using System.Configuration;

public partial class Integrate : System.Web.UI.Page
{
    private string integrateVersion = "2.0.5";
    
    private string sessionId = "";
    private string sessionKey = "";

    protected void Page_Load(object sender, EventArgs e)
    {
        // USER CONFIG
        string accountName = ConfigurationManager.AppSettings["accountName"];
        string manualGroup = ConfigurationManager.AppSettings["manualGroup"];
        string manualName = ConfigurationManager.AppSettings["manualName"];
        string manualRoot = ConfigurationManager.AppSettings["manualRoot"];

        string userIp = Request.ServerVariables["REMOTE_ADDR"];
        string forwardedFor = "";
        if (Request.ServerVariables["HTTP_X_FORWARDED_FOR"] != null) {
          forwardedFor = Request.ServerVariables["HTTP_X_FORWARDED_FOR"];
        }
        
        string requestPage = Request.ServerVariables["REQUEST_URI"];
        // encode request page url
        requestPage = Server.UrlEncode(requestPage);

        string useSession = "yes";

        // session init (set sessionId)
        SetSessionId();

        // check key (set sessionKey)
        SetSessionKey();

        // create request
        string url = "https://www.manula.com/integrate";
        HttpWebRequest request = (HttpWebRequest) HttpWebRequest.Create(url);

        // config
        request.UserAgent = Request.ServerVariables["HTTP_USER_AGENT"];
        request.ContentType = "application/x-www-form-urlencoded";
        if (sessionId != "") request.Headers["Cookie"] = "Manula_App=" + sessionId;

        // POST
        StringBuilder postDataArray = new StringBuilder();
        postDataArray.Append(String.Format("account={0}&", accountName));
        postDataArray.Append(String.Format("group={0}&", manualGroup));
        postDataArray.Append(String.Format("manual_name={0}&", manualName));
        postDataArray.Append(String.Format("manual_root={0}&", manualRoot));
        postDataArray.Append(String.Format("request_page={0}&", requestPage));
        postDataArray.Append(String.Format("user_ip={0}&", userIp));
        postDataArray.Append(String.Format("forwarded_for={0}&", forwardedFor));
        postDataArray.Append(String.Format("can_use_session={0}&", useSession));
        postDataArray.Append(String.Format("_integrate_version={0}-for-ASP", integrateVersion));
        string postData = postDataArray.ToString();

        request.Method = "POST";
        request.ContentLength = postData.Length;
        
        StreamWriter writer = new StreamWriter(request.GetRequestStream(), System.Text.Encoding.ASCII);
        writer.Write(postData);
        writer.Close();

        string result = string.Empty;
        using (HttpWebResponse response = (HttpWebResponse)request.GetResponse()) { 
            using (Stream responseStream = response.GetResponseStream()) {
                using (StreamReader readStream = new StreamReader(responseStream, Encoding.UTF8)) { 
                    result = readStream.ReadToEnd(); 
                }
            }

            // check for (server) sessionid
            for (int i=0; i < response.Headers.Count; ++i) {
                if (response.Headers.Keys[i] == "Set-Cookie" || response.Headers.Keys[i] == "set-cookie") {
                    sessionId = GetSessionId(response.Headers[i]);

                    if (sessionId != "") {
                        // write cookie
                        HttpCookie cookie = new HttpCookie("Manula_App", sessionId); // name, value
                        Response.Cookies.Add(cookie);
                    }
                }
            }
        }
        Response.Write(result);
    }

    protected string GetSessionId(string cookieResponseHeader)
    {
        string sessionId = "";

        string[] cookiesString = cookieResponseHeader.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

        for (int i = 0; i < cookiesString.Length; i++) {
            //Response.Write(cookiesString[i]+"<br>");
            string[] cookiePartsString = cookiesString[i].Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries);
            for (int j = 0; j < cookiePartsString.Length; j++) {
                //Response.Write(cookiePartsString[j] + "<br>");
                string[] valuePairs = cookiePartsString[j].Split(new char[] { '=' }, StringSplitOptions.RemoveEmptyEntries);
                for (int n = 0; n < valuePairs.Length; n++) {
                    //Response.Write(valuePairs[n] + "<br>");
                    // check for Manula_App
                    if (valuePairs[0] != null && valuePairs[0] == "Manula_App") {
                        if (valuePairs[1] != null && valuePairs[1] != "") sessionId = valuePairs[1];
                    }
                }
            }
        }
        
        return sessionId;
    }

    protected void SetSessionId()
    {
        sessionId = "";

        if (Request.QueryString["sid"] != null && Request.QueryString["sid"] != "") {
            sessionId = Request.QueryString["sid"];
            // write cookie
            HttpCookie cookie = new HttpCookie("Manula_App", sessionId); // name, value
            Response.Cookies.Add(cookie);
        } else if (Request.Cookies["Manula_App"] != null  && Request.Cookies["Manula_App"].Value != "") {
            // read cookie
            sessionId = Request.Cookies["Manula_App"].Value;
        }
    }

    protected void SetSessionKey()
    {
        sessionKey = "";

        if (Request.QueryString["key"] != null && Request.QueryString["key"] != "") {
            sessionKey = Request.QueryString["key"];
        }
    }
}
